@@ -3,9 +3,11 @@ from .callablegroup import CallableGroup, CallNoOftenThan
 from .locked_dataset import LockedDataset
 from .locked_structure import LockedStructure
 from .monitor import MonitorList, Monitor, MonitorDict, RMonitor
-from .thread import TerminableThread, Condition, SingleStartThread, call_in_separate_thread
+from .thread import TerminableThread, Condition, SingleStartThread, call_in_separate_thread, \
+    BogusTerminableThread
 from .id_allocator import IDAllocator
 
 __all__ = ['LockedDataset', 'Monitor', 'RMonitor', 'CallableGroup', 'TerminableThread',
            'MonitorDict', 'MonitorList', 'Condition', 'LockedStructure', 'AtomicNumber',
-           'CallNoOftenThan', 'SingleStartThread', 'IDAllocator', 'call_in_separate_thread']
+           'CallNoOftenThan', 'SingleStartThread', 'IDAllocator', 'call_in_separate_thread',
+           'BogusTerminableThread']
