@@ -7,7 +7,7 @@ from satella.exceptions import PreconditionError
 T = tp.TypeVar('T')
 U = tp.TypeVar('U')
 Expression = tp.NewType('Expression', str)
-
+ExcType = tp.Type[Exception]
 
 # noinspection PyPep8Naming
 def _TRUE(x):
@@ -17,11 +17,12 @@ def _TRUE(x):
 Queue = tp.TypeVar('Queue')
 
 
-def queue_get(queue_getter: tp.Callable[[object], Queue], timeout: tp.Optional[float] = None,
-              exception_empty=queue.Empty,
+def queue_get(queue_getter: tp.Union[str, tp.Callable[[object], Queue]], timeout: tp.Optional[float] = None,
+              exception_empty: tp.Union[ExcType, tp.Tuple[ExcType, ...]]=queue.Empty,
               queue_get_method: tp.Callable[[Queue, tp.Optional[float]], tp.Any] =
               lambda x, timeout: x.get(
-                  timeout=timeout)):
+                  timeout=timeout),
+              method_to_execute_on_empty: tp.Optional[tp.Union[str, tp.Callable]] = None):
     """
     A decorator for class methods that consume from a queue.
 
@@ -39,6 +40,10 @@ def queue_get(queue_getter: tp.Callable[[object], Queue], timeout: tp.Optional[f
         empty.
     :param queue_get_method: a method to invoke on this queue. Accepts two arguments - the first
         is the queue, the second is the timeout. It has to follow the type signature given.
+    :param method_to_execute_on_empty: a callable, or a name of the method to be executed (with no
+        arguments other than self) to execute in case queue.Empty was raised. Can be a callable -
+        in that case it should expect no arguments, or can be a string, which will be assumed to be
+        a method name
 
     Use instead of:
 
@@ -71,9 +76,14 @@ def queue_get(queue_getter: tp.Callable[[object], Queue], timeout: tp.Optional[f
             try:
                 que = my_queue_getter(self)
                 item = queue_get_method(que, timeout)
+                return fun(self, item)
             except exception_empty:
-                return
-            return fun(self, item)
+                if method_to_execute_on_empty is not None:
+                    if callable(method_to_execute_on_empty):
+                        method_to_execute_on_empty()
+                    elif isinstance(method_to_execute_on_empty, str):
+                        method = getattr(self, method_to_execute_on_empty)
+                        method()
 
         return inner
 
