@@ -2,10 +2,25 @@ import logging
 import math
 import typing as tp
 
+from satella.coding.decorators import wraps
+from satella.coding.recast_exceptions import rethrow_as
+
 T = tp.TypeVar('T')
 logger = logging.getLogger(__name__)
 
 _SETTABLE_KEYS = {'_Proxy__obj', '_Proxy__wrap_operations'}
+Number = tp.Union[float, int]
+
+
+def wrap_operation(fun):
+    @wraps(fun)
+    def inner(self: 'Proxy', *args, **kwargs):
+        result = fun(self, *args, **kwargs)
+        if self._Proxy__wrap_operations:
+            result = self.__class__(result)
+        return result
+
+    return inner
 
 
 class Proxy(tp.Generic[T]):
@@ -17,8 +32,7 @@ class Proxy(tp.Generic[T]):
     Note that in-place operations will return the Proxy itself, whereas simple addition will shed
     this proxy, returning object wrapped plus something.
 
-    Note that proxies are considered to be the type of the object that they wrap,
-    as well as considered to be of type Proxy.
+    Only __isinstance__ check is not overridden, due to Python limitations.
 
     Please access this object in your descendant classes via
 
@@ -27,16 +41,20 @@ class Proxy(tp.Generic[T]):
     Please note that this class does not overload the descriptor protocol,
     not the pickle interface!
 
-    Note that this overloads both __repr__ and __str__, which may prove confusing.
+    Note that this overloads __repr__, __str__ and __dir__, which may prove confusing.
     Handle this in your descendant classes.
 
+    If wrap_operations is set, the following code will be executed on the result:
+
+    >>> a = a.__add__(b)
+    >>> return self.__class__(a)
+
+    Wrapped operations include all arithmetic operations and bitwise operations,
+    except for divmod, but including concat, rounding, truncing and ceiling.
+
     :param object_to_wrap: object to wrap
     :param wrap_operations: whether results of operations returning something else should be
-        also proxied. This will be done by the following code:
-        >>> a = a.__add__(b)
-        >>> return self.__class__(a)
-        Wrapped operations include ONLY add, sub, mul, all kinds of div.
-        If you want logical operations wrapped, file an issue.
+        also proxied.
     """
     __slots__ = ('__obj', '__wrap_operations')
 
@@ -50,13 +68,13 @@ class Proxy(tp.Generic[T]):
     def __getitem__(self, item):
         return self.__obj[item]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key, value) -> None:
         self.__obj[key] = value
 
-    def __delitem__(self, key):
+    def __delitem__(self, key) -> None:
         del self.__obj[key]
 
-    def __setattr__(self, key, value):
+    def __setattr__(self, key, value) -> None:
         if key in _SETTABLE_KEYS:
             super().__setattr__(key, value)
         else:
@@ -65,83 +83,83 @@ class Proxy(tp.Generic[T]):
     def __getattr__(self, item):
         return getattr(self.__obj, item)
 
-    def __delattr__(self, item):
+    def __delattr__(self, item) -> None:
         delattr(self.__obj, item)
 
     def __int__(self) -> int:
         return int(self.__obj)
 
+    @rethrow_as(AttributeError, TypeError)
     def __float__(self) -> float:
         return float(self.__obj)
 
+    @rethrow_as(AttributeError, TypeError)
     def __complex__(self) -> complex:
         return complex(self.__obj)
 
     def __str__(self) -> str:
         return str(self.__obj)
 
+    @wrap_operation
     def __add__(self, other):
-        result = self.__obj + other
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+        return self.__obj + other
 
     def __iadd__(self, other) -> 'Proxy':
         self.__obj += other
         return self
 
+    @wrap_operation
     def __sub__(self, other):
-        result = self.__obj - other
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+        return self.__obj - other
 
     def __isub__(self, other) -> 'Proxy':
         self.__obj -= other
         return self
 
+    @wrap_operation
     def __mul__(self, other):
-        result = self.__obj * other
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+        return self.__obj * other
 
-    def __divmod__(self, other):
-        result = divmod(self.__obj, other)
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+    def __divmod__(self, other) -> tp.Tuple[Number, Number]:
+        return divmod(self.__obj, other)
 
+    @wrap_operation
     def __floordiv__(self, other):
-        result = self.__obj // other
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+        return self.__obj // other
 
+    def __rdivmod__(self, other):
+        return divmod(other, self.__obj)
+
+    @wrap_operation
     def __truediv__(self, other):
-        result = self.__obj / other
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+        return self.__obj / other
 
     def __imul__(self, other) -> 'Proxy':
         self.__obj *= other
         return self
 
     def __itruediv__(self, other) -> 'Proxy':
-        self.obj /= other
+        self.__obj /= other
         return self
 
     def __ifloordiv__(self, other) -> 'Proxy':
-        self.obj //= other
+        self.__obj //= other
         return self
 
+    @wrap_operation
+    def __rshift__(self, other):
+        return self.__obj >> other
+
+    @wrap_operation
+    def __lshift__(self, other):
+        return self.__obj << other
+
     def __ilshift__(self, other) -> 'Proxy':
-        self.obj <<= other
+        self.__obj <<= other
         return self
 
     def __irshift__(self, other) -> 'Proxy':
-        self.obj >>= other
+        self.__obj >>= other
         return self
 
     def __iter__(self) -> tp.Iterator:
@@ -159,11 +177,14 @@ class Proxy(tp.Generic[T]):
     def __eq__(self, other) -> bool:
         return self.__obj == other
 
+    def __ne__(self, other) -> bool:
+        return self.__obj != other
+
     def __or__(self, other):
-        return self.__obj or other
+        return self.__obj | other
 
     def __and__(self, other):
-        return self.__obj and other
+        return self.__obj & other
 
     def __le__(self, other) -> bool:
         return self.__obj <= other
@@ -186,6 +207,13 @@ class Proxy(tp.Generic[T]):
     def __repr__(self) -> str:
         return repr(self.__obj)
 
+    def __oct__(self) -> str:
+        return oct(self.__obj)
+
+    def __hex__(self) -> str:
+        return hex(self.__obj)
+
+    @wrap_operation
     def __abs__(self):
         return abs(self.__obj)
 
@@ -198,18 +226,63 @@ class Proxy(tp.Generic[T]):
     def __next__(self):
         return next(self.__obj)
 
+    @wrap_operation
     def __xor__(self, other):
         return self.__obj ^ other
 
+    @wrap_operation
+    def __radd__(self, other):
+        return other + self.__obj
+
+    @wrap_operation
+    def __rsub__(self, other):
+        return other - self.__obj
+
+    @wrap_operation
+    def __rmul__(self, other):
+        return other * self.__obj
+
+    @wrap_operation
+    def __rtruediv__(self, other):
+        return other / self.__obj
+
+    @wrap_operation
+    def __rfloordiv__(self, other):
+        return other // self.__obj
+
+    @wrap_operation
+    def __rlshift__(self, other):
+        return other << self.__obj
+
+    @wrap_operation
+    def __rrshift__(self, other):
+        return other >> self.__obj
+
+    @wrap_operation
+    def __rpow__(self, other):
+        return other ** self.__obj
+
+    @wrap_operation
+    def __ror__(self, other):
+        return other | self.__obj
+
+    @wrap_operation
+    def __rand__(self, other):
+        return other & self.__obj
+
+    @wrap_operation
+    def __rxor__(self, other):
+        return other ^ self.__obj
+
     def __ior__(self, other) -> 'Proxy':
         self.__obj |= other
         return self
 
-    def __iand__(self, other):
+    def __iand__(self, other) -> 'Proxy':
         self.__obj &= other
         return self
 
-    def __ixor__(self, other):
+    def __ixor__(self, other) -> 'Proxy':
         self.__obj ^= other
         return self
 
@@ -228,84 +301,87 @@ class Proxy(tp.Generic[T]):
     def __reversed__(self):
         return reversed(self.__obj)
 
+    @wrap_operation
     def __pow__(self, power, modulo=None):
-        if self.__wrap_operations:
-            result = self.__class__(self.__obj.__pow__(power, modulo))
-        else:
-            result = self.__obj.__pow__(power, modulo)
-        return result
+        return pow(self.__obj, power, modulo)
 
     def __ipow__(self, other) -> 'Proxy':
-        self.__obj.__ipow__(other)
+        self.__obj **= other
         return self
 
+    @wrap_operation
     def __neg__(self):
-        if self.__wrap_operations:
-            result = self.__class__(self.__obj.__neg__())
-        else:
-            result = self.__obj.__neg__()
-        return result
+        return -self.__obj
 
+    @wrap_operation
     def __pos__(self):
-        if self.__wrap_operations:
-            result = self.__class__(self.__obj.__pow__())
-        else:
-            result = self.__obj.__pow__()
-        return result
+        return +self.__obj
 
+    @wrap_operation
     def __invert__(self):
-        if self.__wrap_operations:
-            result = self.__class__(self.__obj.__invert__())
-        else:
-            result = self.__obj.__invert__()
-        return result
+        return ~self.__obj
+
+    @wrap_operation
+    def __matmul__(self, other):
+        return self.__obj @ other
+
+    def __imatmul__(self, other):
+        self.__obj @= other
+        return self
+
+    @wrap_operation
+    def __rmatmul__(self, other):
+        return other @ self.__obj
 
     def __index__(self) -> int:
         return self.__obj.__index__()
 
-    def __round__(self, n=None):
-        result = self.__obj.__round__(n)
-        if self.__wrap_operations:
-            result = self.__class__(result)
-
-        return result
+    @wrap_operation
+    @rethrow_as(AttributeError, TypeError)
+    def __round__(self, n: int = 0):
+        return round(self.__obj, n)
 
+    @wrap_operation
+    @rethrow_as(AttributeError, TypeError)
     def __trunc__(self):
-        result = self.__obj.__trunc__()
-        if self.__wrap_operations:
-            result = self.__class__(result)
-
+        if hasattr(self.__obj, '__trunc__'):
+            result = self.__obj.__trunc__()
+        else:
+            result = math.trunc(self.__obj)
         return result
 
+    @wrap_operation
+    @rethrow_as(AttributeError, TypeError)
     def __floor__(self):
         if hasattr(self.__obj, '__floor__'):
             result = self.__obj.__floor__()
         else:
             result = math.floor(self.__obj)
 
-        if self.__wrap_operations:
-            result = self.__class__(result)
         return result
 
+    @wrap_operation
+    @rethrow_as(AttributeError, TypeError)
     def __ceil__(self):
-        result = self.__obj.__ceil__()
-
-        if self.__wrap_operations:
-            result = self.__class__(result)
+        if hasattr(self.__obj, '__ceil__'):
+            result = self.__obj.__ceil__()
+        else:
+            result = math.ceil(self.__obj)
         return result
 
     def __dir__(self) -> tp.Iterable[str]:
         return dir(self.__obj)
 
+    @wrap_operation
+    @rethrow_as(AttributeError, TypeError)
     def __concat__(self, other):
-        result = self.__obj.__concat__(other)
-        if self.__wrap_operations:
-            result = self.__class__(result)
-        return result
+        return self.__obj.__concat__(other)
 
+    @rethrow_as(AttributeError, TypeError)
     def __iconcat__(self, other) -> 'Proxy':
         self.__obj.__iconcat__(other)
         return self
 
+    @rethrow_as(AttributeError, TypeError)
     def __length_hint__(self):
         return self.__obj.__length_hint__()
